package list

import (
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"path/filepath"
	"strings"
	"text/template"
	"time"

	"github.com/remogatto/cloud"

	karmen_client "gogs.carducci-dante.gov.it/karmen/client"
	"gogs.carducci-dante.gov.it/karmen/core/config"
	"gogs.carducci-dante.gov.it/karmen/core/orm"
	"gogs.carducci-dante.gov.it/karmen/util/fileutil"
	"gogs.carducci-dante.gov.it/karmen/util/libreoffice"
	"gogs.carducci-dante.gov.it/karmen/util/pandoc"
	tpl_util "gogs.carducci-dante.gov.it/karmen/util/template"
)

type ListGenerator struct {
	Config *config.ConfigT
}

func NewListGenerator(config *config.ConfigT) *ListGenerator {
	return &ListGenerator{config}
}

var funcMap template.FuncMap = template.FuncMap{
	"comma":        comma,
	"groupClasses": groupClasses,
	"abbrev":       abbrev,
	"nbsp":         nbsp,
}

func (generator *ListGenerator) generate(outputPath string, teachers []*orm.Teacher, funcMap template.FuncMap) {
	filename := filepath.Join(outputPath, "elenco_docenti")
	tpl, err := tpl_util.LoadTextTemplate("./generator/generators/list/list.tpl.md", funcMap)
	if err != nil {
		panic(err)
	}

	f, err := os.Create(filename + ".md")
	if err != nil {
		panic(err)
	}
	defer f.Close()

	err = tpl.Execute(f, teachers)
	if err != nil {
		panic(err)
	}

	odtFilename := fileutil.ReplaceExt(filename, "odt")

	log.Println("Generate", odtFilename)

	if err := pandoc.Convert(filename+".md", odtFilename, "--data-dir", "./generator/generators/list/"); err != nil {
		panic(err)
	}

	if err := libreoffice.Convert(filename+".odt", "pdf", "--outdir", outputPath); err != nil {
		panic(err)
	}

}

func comma(classes []*orm.Class) string {
	var names []string
	for _, c := range classes {
		names = append(names, c.Name)
	}
	return strings.Join(names, ",")
}

func abbrev(name string) string {
	var result string
	splits := strings.Split(name, " ")
	for i := len(splits) - 1; i >= 0; i-- {
		if i == len(splits)-1 {
			result += splits[i] + " "
			continue
		}
		result += strings.ToUpper(string(splits[i][0]) + ".")
	}
	return result
}

func nbsp(num int, text string) string {
	return text + strings.Repeat("&nbsp;", num)
}

func groupClasses(classes []*orm.Class) string {
	var groups []string

	groupByAddresses := make(map[string][]string)
	addressesAbbrev := map[string]string{
		"Linguistico":       "LIN",
		"Classico":          "CLA",
		"Musicale":          "M",
		"Economico sociale": "ES",
		"Scienze umane":     "SU",
	}
	for _, c := range classes {
		groupByAddresses[c.Field] = append(groupByAddresses[c.Field], fmt.Sprintf("%d%s", c.Year, c.Section))
	}

	for address, classes := range groupByAddresses {
		var group string
		if len(classes) > 1 {
			group = fmt.Sprintf("[%s]%s", strings.Join(classes, ","), addressesAbbrev[address])
		} else {
			group = fmt.Sprintf("%s %s", strings.Join(classes, ","), addressesAbbrev[address])
		}
		groups = append(groups, group)
	}
	return strings.Join(groups, ",")
}

func (generator *ListGenerator) Run(jobId uint) {
	ncClient, err := cloud.Dial(
		config.Config.Cloud.Url,
		config.Config.Cloud.Username,
		config.Config.Cloud.Password,
	)
	if err != nil {
		panic(err)
	}

	log.Printf("Connecting to karmen...")
	client, err := karmen_client.Dial(
		"http://localhost:3000",
		"admin",
		"admin",
	)
	if err != nil {
		log.Println(err)
	}

	job, err := client.GetJob(jobId)
	if err != nil {
		panic(err)
	}

	job.Start = time.Now()
	err = client.UpdateJob(job)
	if err != nil {
		panic(err)
	}

	teachers, err := client.GetTeachers()
	if err != nil {
		panic(err)
	}

	outputPath := filepath.Join(config.Config.Documents.OutputPath, fmt.Sprintf("%d/%d", job.DocumentID, job.ID))

	if err := os.MkdirAll(outputPath, 0777); err != nil {
		panic(err)
	}

	generator.generate(outputPath, teachers, funcMap)

	job.End = time.Now()

	job.Files = append(job.Files, &orm.File{Path: "elenco_docenti.pdf"})

	err = client.UpdateJob(job)
	if err != nil {
		panic(err)
	}

	log.Println("Uploading files to the cloud...")
	src, err := ioutil.ReadFile(filepath.Join(outputPath, "elenco_docenti.pdf"))
	if err != nil {
		panic(err)
	}
	err = ncClient.Upload(src, "Documents/elenco_docenti.pdf")
	if err != nil {
		panic(err)
	}
}
